@@ -305,10 +305,52 @@ class NeoBeeShell:
 
         return info
 
+    def _write_int(self, value: int, index: int = 0) -> None:
+        iValue = int(value)
+        NEG_FLAG = value < 0
+        if NEG_FLAG:
+            iValue = -1 * iValue
+
+        self[index] = (iValue >> 24) & 0xFF
+        self[index + 1] = (iValue >> 16) & 0xFF
+        self[index + 2] = (iValue >> 8) & 0xFF
+        self[index + 3] = (iValue) & 0xFF
+
+        # set negative flag
+        if NEG_FLAG:
+            self[index] |= 0b10000000
+
+    def _write_float(self, value: float, index: int = 0) -> None:
+        self._write_int(int(value * 100))
+
+    def _read_ufloat(self, index: int = 0) -> float:
+        return self._read_uint(index) / 100.0
+
     def _read_float(self, index: int = 0) -> float:
-        return (
+        """
+        Decodes a four bytes encoded signed float
+        value.
+        The highest bit of the first byte indicates
+        wether the value is negative (1) or not (0).
+        """
+        return self._read_int(index) / 100.0
+
+    def _read_int(self, index: int = 0) -> int:
+        NEG_FLAG = self[0] & 0b10000000
+        # Clear the negative flag
+        value = (
+            ((self[index] & 0b01111111) << 24)
+            | (self[index + 1] << 16)
+            | (self[index + 2] << 8)
+            | self[index + 3]
+        )
+        return int((-1 * value) if NEG_FLAG else value)
+
+    def _read_uint(self, index: int = 0) -> int:
+        value = (
             (self[index] << 24) | (self[index + 1] << 16) | (self[index + 2] << 8) | self[index + 3]
-        ) / 100.0
+        )
+        return int(value)
 
     @property
     def method(self) -> RequestMethod:
@@ -344,7 +386,7 @@ class NeoBeeShell:
         self._send()
 
         if self.status == StatusCode.OK:
-            return ((self[0] << 24) | (self[1] << 16) | (self[2] << 8) | self[3]) / 100
+            return self._read_uint()
         elif self.status == StatusCode.NOT_FOUND:
             return None
         else:
@@ -355,17 +397,10 @@ class NeoBeeShell:
         if not self.connected:
             raise NotConnectedError()
 
-        if value <= 0:
-            raise BadRequestError("Offset must be a positive value.")
-
         self._clearbuffer()
         self.method = RequestMethod.PUT
         self.command = CmdCode.SCALE_OFFSET
-        iValue = int(value * 100)
-        self[0] = (iValue >> 24) & 0xFF
-        self[1] = (iValue >> 16) & 0xFF
-        self[2] = (iValue >> 8) & 0xFF
-        self[3] = (iValue) & 0xFF
+        self._write_int(value)
         self._send()
 
     @property
@@ -381,7 +416,7 @@ class NeoBeeShell:
         self.command = CmdCode.SCALE_FACTOR
         self._send()
         if self.status == StatusCode.OK:
-            return ((self[0] << 24) | (self[1] << 16) | (self[2] << 8) | self[3]) / 100
+            return self._read_ufloat()
         elif self.status == StatusCode.NOT_FOUND:
             return None
         else:
@@ -392,17 +427,13 @@ class NeoBeeShell:
         if not self.connected:
             raise NotConnectedError("Not connected")
 
-        if value <= 0:
-            raise BadRequestError("Factor must be a positive value.")
+        # if value <= 0:
+        #    raise BadRequestError("Factor must be a positive value.")
 
         self._clearbuffer()
         self.method = RequestMethod.PUT
         self.command = CmdCode.SCALE_FACTOR
-        iValue = int(value * 100)
-        self[0] = (iValue >> 24) & 0xFF
-        self[1] = (iValue >> 16) & 0xFF
-        self[2] = (iValue >> 8) & 0xFF
-        self[3] = (iValue) & 0xFF
+        self._write_float(value)
         self._send()
 
     @property
@@ -591,7 +622,7 @@ class NeoBeeShell:
         self.method = RequestMethod.GET
         self.command = CmdCode.GET_TEMPERATURE
         self._send()
-        return ((self[0] << 24) | (self[1] << 16) | (self[2] << 8) | self[3]) / 100
+        return self._read_float()
 
     def tare(self, nr_times: int):
         """
@@ -609,8 +640,8 @@ class NeoBeeShell:
         self.command = CmdCode.TARE
         self[0] = nr_times & 0xFF
         self._send()
-        offset = ((self[0] << 24) | (self[1] << 16) | (self[2] << 8) | self[3]) / 100
-        factor = ((self[4] << 24) | (self[5] << 16) | (self[6] << 8) | self[7]) / 100
+        offset = self._read_int()
+        factor = self._read_float(index=4)
         return (offset, factor)
 
     def calibrate(self, ref_weight: int, count: int):
@@ -637,8 +668,8 @@ class NeoBeeShell:
         self[1] = ref_weight & 0xFF
         self[2] = count & 0xFF
         self._send()
-        offset = ((self[0] << 24) | (self[1] << 16) | (self[2] << 8) | self[3]) / 100
-        factor = ((self[4] << 24) | (self[5] << 16) | (self[6] << 8) | self[7]) / 100
+        offset = self._read_uint()
+        factor = self._read_ufloat(index=4)
         return (offset, factor)
 
     @property
@@ -740,7 +771,7 @@ class NeoBeeShell:
         self.command = CmdCode.GET_WEIGHT
         self[0] = 1
         self._send()
-        return ((self[0] << 24) | (self[1] << 16) | (self[2] << 8) | self[3]) / 100
+        return self._read_float()
 
     def reset(self):
         self._clearbuffer()
